"""
Parses whisper options.
"""

import re
import subprocess
from typing import Any

from transcribe_anything import logger
from transcribe_anything.whisper import get_environment

PATTERN = r"\s+\[--(.*?)\]"


def _parse_item(item: str) -> tuple[str, Any]:
    """Parses item into a key,value pair."""
    value: Any = None
    key, value = item.split(" ", 1)
    if "{" in value:
        value = value.replace("{", "").replace("}", "").split(",")
        value = [v.strip() for v in value if v.strip()]
    return (key, value)


def parse_whisper_options() -> dict:
    """Parses the whisper options."""
    env = get_environment()
    stdout = subprocess.check_output(
        "whisper --help", shell=True, universal_newlines=True, encoding="utf-8", env=env
    )
    lines = stdout.splitlines()
    data = {}
    for line in lines:
        items = re.findall(PATTERN, line)
        if not items:
            continue
        for item in items:
            try:
                key, value = _parse_item(item)
                data[key] = value
            except Exception as exc:  # pylint: disable=broad-except
                logger.log_error(f"Failed to parse {items}: {exc}")
                continue
    return data


if __name__ == "__main__":
    from pprint import pprint

    pprint(parse_whisper_options())



"""
Tests transcribe_anything
"""

# pylint: disable=bad-option-value,useless-option-value,no-self-use,protected-access,R0801
# flake8: noqa E501

import os
import shutil
import unittest
from pathlib import Path

from transcribe_anything.insanely_fast_whisper import (
    CudaInfo,
    get_cuda_info,
    has_nvidia_smi,
    run_insanely_fast_whisper,
)
from transcribe_anything.util import is_mac_arm

HERE = Path(os.path.abspath(os.path.dirname(__file__)))
LOCALFILE_DIR = HERE / "localfile"
TESTS_DATA_DIR = LOCALFILE_DIR / "text_video_insane" / "en"
TEST_WAV = LOCALFILE_DIR / "video.wav"

CAN_RUN_TEST = has_nvidia_smi() or is_mac_arm()


class InsanelFastWhisperTester(unittest.TestCase):
    """Tester for transcribe anything."""

    @unittest.skipUnless(CAN_RUN_TEST, "No GPU detected")
    def test_local_file(self) -> None:
        """Check that the command works on a local file."""
        shutil.rmtree(TESTS_DATA_DIR, ignore_errors=True)
        run_insanely_fast_whisper(
            input_wav=TEST_WAV,
            model="small",
            output_dir=TESTS_DATA_DIR,
            task="transcribe",
            language="en",
        )

    @unittest.skipUnless(CAN_RUN_TEST, "No GPU detected")
    def test_cuda_info(self) -> None:
        """Check that the command works on a local file."""
        cuda_info0 = get_cuda_info()
        out = cuda_info0.to_json_str()
        cuda_info1 = CudaInfo.from_json_str(out)
        print(out)
        self.assertEqual(cuda_info0, cuda_info1)


if __name__ == "__main__":
    unittest.main()



# pylint: skip-file

"""
Queries the system for CUDA devices and returns a json string with the information.
This is meant to be run under a "isolated-environment".
"""

import json
import shutil
import sys
from dataclasses import asdict, dataclass, fields
from typing import Any


@dataclass
class CudaDevice:
    """Data class to hold CUDA device information."""

    name: str
    vram: int  # VRAM in bytes
    multiprocessors: int  # Number of multiprocessors
    device_id: int

    def __str__(self):
        return (
            f"{self.name} - VRAM: {self.vram / (1024 ** 3):.2f} GB, "
            f"Multiprocessors: {self.multiprocessors}"
        )

    def to_json(self) -> dict[str, str | int]:
        """Returns a dictionary representation of the object."""
        return asdict(self)

    @staticmethod
    def from_json(json_data: dict[str, str | int]) -> "CudaDevice":
        """Returns a CudaDevice object from a dictionary."""
        return CudaDevice(**json_data)  # type: ignore


@dataclass
class CudaInfo:
    """Cuda info."""

    cuda_available: bool
    num_cuda_devices: int
    cuda_devices: list[CudaDevice]

    def to_json_str(self) -> str:
        """Returns json str."""
        # Convert dataclass to dictionary for serialization
        data = self.to_json()
        return json.dumps(data, indent=4, sort_keys=True)

    def to_json(self) -> dict[str, Any]:
        """Returns a dictionary representation of the object."""
        out = {}
        for field in fields(self):
            out[field.name] = getattr(self, field.name)
            if field.name == "cuda_devices":
                out[field.name] = [device.to_json() for device in out[field.name]]
        return out

    @staticmethod
    def from_json_str(json_str: str) -> "CudaInfo":
        """Loads from json str and returns a CudaInfo object."""
        assert json_str is not None, "Expected json_str to be set, but was instead None"
        data = json.loads(json_str)
        cuda_devices_data = data.get("cuda_devices", [])
        cuda_devices = [CudaDevice(**device) for device in cuda_devices_data]
        return CudaInfo(data["cuda_available"], data["num_cuda_devices"], cuda_devices)


def cuda_cards_available() -> CudaInfo:
    """
    Returns a CudaInfo object with information about the CUDA cards,
    ordered by VRAM and multiprocessors.
    """
    # Have to import here, since others will import CudaDevice and CudaInfo.
    if shutil.which("nvidia-smi") is None:
        return CudaInfo(False, 0, [])
    import torch  # type: ignore

    if torch.cuda.is_available():
        devices = [
            CudaDevice(
                name=torch.cuda.get_device_name(i),
                vram=torch.cuda.get_device_properties(i).total_memory,
                multiprocessors=torch.cuda.get_device_properties(
                    i
                ).multi_processor_count,
                device_id=i,
            )
            for i in range(torch.cuda.device_count())
        ]
        # Sort devices by VRAM and then by number of multiprocessors in descending order
        devices.sort(key=lambda x: (x.vram, x.multiprocessors), reverse=True)
        return CudaInfo(True, len(devices), devices)
    return CudaInfo(False, 0, [])


def main() -> int:
    """Returns 0 if cuda is available, 1 otherwise."""
    cuda_info = cuda_cards_available()
    json_str = cuda_info.to_json_str()
    assert json_str is not None, "Expected json_str to be set, but was instead None"
    print(json_str)
    if cuda_info.cuda_available:
        return 0
    return 1


if __name__ == "__main__":
    sys.exit(main())



"""
Tests transcribe_anything
"""

# pylint: disable=bad-option-value,useless-option-value,no-self-use,protected-access
# flake8: noqa E501

import os
import shutil
import subprocess
import unittest

HERE = os.path.abspath(os.path.dirname(__file__))
TESTS_DATA_DIR = os.path.join(HERE, "test_data")
LOCALFILE_DIR = os.path.join(HERE, "localfile")


class TranscribeAnythingTester(unittest.TestCase):
    """Tester for transcribe anything."""

    def test_fetch_command_help(self) -> None:
        """Check that the command is installed by the setup process."""
        subprocess.check_output(["transcribe_anything", "-h"])

    def test_local_file(self) -> None:
        """Check that the command works on a local file."""
        shutil.rmtree(TESTS_DATA_DIR, ignore_errors=True)
        subprocess.check_output(
            ["transcribe_anything", "video.mp4", "--language", "en", "--model", "tiny"],
            cwd=LOCALFILE_DIR,
        )

    def test_initial_prompt(self) -> None:
        """Check that the command works on a local file."""
        shutil.rmtree(TESTS_DATA_DIR, ignore_errors=True)
        subprocess.check_output(
            [
                "transcribe_anything",
                "video.mp4",
                "--language",
                "en",
                "--model",
                "tiny",
                "--initial_prompt",
                '"nervous"',
            ],
            cwd=LOCALFILE_DIR,
        )
        # Expect that when the language is explicitly set that the output file ends up in
        # that language folder.
        expected_output = os.path.join(LOCALFILE_DIR, "text_video", "en", "out.txt")
        self.assertTrue(
            os.path.exists(expected_output),
            f"Path {expected_output} doesn't exist, instead it was {os.listdir(LOCALFILE_DIR)}",  # pylint: disable=line-too-long
        )

    def test_fetch_command_installed(self) -> None:
        """Check that the command works on a live short video."""
        shutil.rmtree(TESTS_DATA_DIR, ignore_errors=True)
        cmd = (
            "transcribe_anything https://www.youtube.com/watch?v=DWtpNPZ4tb4"
            f" --model tiny --output_dir {TESTS_DATA_DIR}"
        )
        rtn_val = subprocess.call(cmd, shell=True)
        self.assertEqual(rtn_val, 0, "command failed")
        expected_paths = [
            TESTS_DATA_DIR,
            # os.path.join(TESTS_DATA_DIR, "out.mp3"),
            os.path.join(TESTS_DATA_DIR, "out.txt"),
            os.path.join(TESTS_DATA_DIR, "out.srt"),
            os.path.join(TESTS_DATA_DIR, "out.vtt"),
        ]
        for expected_path in expected_paths:
            self.assertTrue(
                os.path.exists(expected_path),
                f"expected path {expected_path} not found",
            )


if __name__ == "__main__":
    unittest.main()



"""
Tests transcribe_anything
"""

# pylint: disable=bad-option-value,useless-option-value,no-self-use,protected-access

import os
import shutil
import subprocess
import unittest

HERE = os.path.abspath(os.path.dirname(__file__))
LOCALFILE_DIR = os.path.join(HERE, "localfile")
TESTS_DATA_DIR = os.path.join(LOCALFILE_DIR, "text_video", "en")


class TranscribeAnythingTester(unittest.TestCase):
    """Tester for transcribe anything."""

    def test_local_file(self) -> None:
        """Check that the command works on a local file."""
        shutil.rmtree(TESTS_DATA_DIR, ignore_errors=True)
        try:
            subprocess.check_output(
                [
                    "transcribe_anything",
                    "video.mp4",
                    "--language",
                    "en",
                    "--model",
                    "tiny",
                ],
                cwd=LOCALFILE_DIR,
            )
        except subprocess.CalledProcessError as e:  # pylint: disable=R0801
            print(e.output)
            raise e


if __name__ == "__main__":
    unittest.main()



import sys
import os

from shutil import rmtree
from setuptools import find_packages, setup, Command

# The directory containing this file
HERE = os.path.dirname(__file__)

NAME = "transcribe-anything"
DESCRIPTION = (
    "Uses whisper AI to transcribe speach from video and audio files. "
    "Also accepts urls for youtube, rumble, bitchute, clear file, etc."
)
URL = "https://github.com/zackees/transcribe-anything"
EMAIL = "dont@email.me"
AUTHOR = "Zach Vorhies"
REQUIRES_PYTHON = ">=3.10.0"
VERSION = "2.7.24"

# The text of the README file
with open(os.path.join(HERE, "README.md"), encoding="utf-8", mode="r") as fd:
    README = fd.read()


def parse_requirements(filename):
    """load requirements from a pip requirements file"""
    with open(filename, encoding="utf-8", mode="rt") as fd:
        lines = [line.strip() for line in fd.readlines() if line.strip()]
    lines = [line.split("#")[0].strip() for line in lines]
    lines = [line for line in lines if line]
    return lines


REQUIREMENTS = parse_requirements(os.path.join(HERE, "requirements.txt"))


class UploadCommand(Command):
    """Support setup.py upload."""

    description = "Build and publish the package."
    user_options = []

    @staticmethod
    def status(s):
        pass

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status("Removing previous builds…")
            rmtree(os.path.join(HERE, "dist"))
        except OSError:
            pass

        self.status("Building Source and Wheel (universal) distribution…")
        os.system("{0} setup.py sdist bdist_wheel --universal".format(sys.executable))

        self.status("Uploading the package to PyPI via Twine…")
        os.system("twine upload dist/*")

        self.status("Pushing git tags…")
        os.system("git tag v{0}".format(VERSION))
        os.system("git push --tags")

        sys.exit()


setup(
    name=NAME,
    python_requires=REQUIRES_PYTHON,
    version=VERSION,
    description=DESCRIPTION,
    keywords="transcription translation speech-to-text speech-recognition command-line-tool ai whisper ml anything",
    long_description=README,
    long_description_content_type="text/markdown",
    url=URL,
    author="Zach Vorhies",
    author_email="dont@email.me",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Environment :: Console",
        "Programming Language :: Python :: 3.9",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX",
        "Operating System :: MacOS :: MacOS X",
        "Environment :: Console",
    ],
    install_requires=REQUIREMENTS,
    entry_points={
        "console_scripts": [
            "transcribe_anything = transcribe_anything.cmd:main",
            "transcribe-anything = transcribe_anything.cmd:main",
        ],
    },
    packages=find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    package_data={},
    include_package_data=True,
    cmdclass={
        "upload": UploadCommand,
    },
)
